package com.yizhi.application.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.constant.EnrollConstant;
import com.yizhi.application.domain.Enroll;
import com.yizhi.application.domain.EnrollRecord;
import com.yizhi.application.mapper.EnrollMapper;
import com.yizhi.application.mapper.EnrollRecordMapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.service.EnrollService;
import com.yizhi.application.util.EnrollEvenSendMessage;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.enroll.application.enums.EvenType;
import com.yizhi.enroll.application.vo.GetEnrollVO;
import com.yizhi.enroll.application.vo.ProjectAccountVO;
import com.yizhi.enroll.application.vo.ReturnEnrollVO;
import com.yizhi.training.application.feign.TrainingProjectClient;
import com.yizhi.training.application.vo.domain.TrainingProjectVo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 报名主体表，入口在培训项目 服务实现类
 * </p>
 *
 * @author bob123
 * @since 2018-03-07
 */
@Service
@Transactional
public class EnrollServiceImpl extends ServiceImpl<EnrollMapper, Enroll> implements EnrollService {

    @Autowired
    EnrollMapper enrollMapper;
    @Autowired
    EnrollRecordMapper enrollRecordMapper;
    @Autowired
    IdGenerator idGenerator;
    @Autowired
    private TaskExecutor taskExecutor;
    @Autowired
    private TrainingProjectClient trainingProjectClient;
    @Autowired
    private EnrollEvenSendMessage enrollEvenSendMessage;

    private static final Logger logger = LoggerFactory.getLogger(EnrollServiceImpl.class);

    @Override
    public Long insertEnroll(Enroll enroll) {

        RequestContext context = ContextHolder.get();
        // 校验参数 报名配置的参数
        boolean valid = checkEnrollParam(enroll,context);
        if (!valid){
            return -1L;
        }
        Integer enablePay = enroll.getEnablePay();
        if (null != enablePay && enablePay.equals(1)){
            //开启付费报名；审核自动设置为否
            enroll.setNeedAudit(0);
        }

        //关闭报名再开启前端会调新增接口,旧报名删除
        if (null != enroll && null != enroll.getTrainingProjectId()) {
            Enroll enroll1 = new Enroll();
            enroll1.setTrainingProjectId(enroll.getTrainingProjectId());
            enrollMapper.delete(new EntityWrapper<>(enroll1));
        }
        enroll.setId(idGenerator.generate());
        enroll.setCompanyId(context.getCompanyId());
        enroll.setOrgId(context.getOrgId());
        enroll.setSiteId(context.getSiteId());
        enroll.setCreateById(context.getAccountId());
        enroll.setCreateByName(context.getAccountName());
        enroll.setCreateTime(new Date());
        enroll.setUpdateById(context.getAccountId());
        enroll.setUpdateByName(context.getAccountName());
        enroll.setUpdateTime(new Date());

        Integer insert = enrollMapper.insert(enroll);
        if (insert > 0) {
            TrainingProjectVo one = trainingProjectClient.getOne(enroll.getTrainingProjectId());
            try {
                //保存了报名  就发消息提醒
                if (one != null) {
                    taskExecutor.asynExecute(new AbstractTaskHandler() {
                        @Override
                        public void handle() {
                            enrollEvenSendMessage.evenSendMessage(one, enroll.getId(),enroll.getEndTime(), null, EvenType.ENROLL_START);
                        }
                    });
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        return enroll.getId();
    }

    /**
     * 校验报名设置的入参
     * @param enroll 入参
     * @return true:符合要求；false:不符合要求
     */
    private boolean checkEnrollParam(Enroll enroll,RequestContext context) {
        Integer enablePay = enroll.getEnablePay();
        Integer actualPrice = enroll.getActualPrice();
        Date startTime = enroll.getStartTime();
        Date endTime = enroll.getEndTime();
        Integer personLimitNum = enroll.getPersonLimitNum();
        if (null == enablePay || null == startTime || null == endTime
                || (null != personLimitNum && personLimitNum.compareTo(0) < 0)
        ){
            logger.error("项目报名参数设置错误：参数为空或参数为负数.companyId = {} ,siteId = {}, 入参 = {}",context.getCompanyId(),context.getSiteId(), JSONObject.toJSONString(enroll));
            return false;
        }
        if (enablePay.equals(1)){
            if (null == actualPrice || actualPrice.compareTo(0) < 0 ){
                logger.error("项目报名参数设置错误：开启付费,实际价格为空! companyId = {} ,siteId = {}, 入参 = {}",context.getCompanyId(),context.getSiteId(), JSONObject.toJSONString(enroll));
                return false;
            }
        }
        return true;
    }

    @Override
    public Long updateEnroll(Enroll enroll) {
        RequestContext context = ContextHolder.get();
        Long enrollId = enroll.getId();
        if (null == enrollId || enrollId.compareTo(0L) < 0){
            logger.error("修改项目报名错误：id参数为空.companyId = {} ,siteId = {}, 入参 = {}",context.getCompanyId(),context.getSiteId(), JSONObject.toJSONString(enroll));
            return -1L;
        }
        boolean result = checkEnrollParam(enroll, context);
        if (!result){
            return -1L;
        }
        Enroll enrollEntity = enrollMapper.selectById(enrollId);
        if (null == enroll){
            logger.error("修改项目报名错误：根据enrollId未查询到报名信息.companyId = {} ,siteId = {}, 入参 = {}",context.getCompanyId(),context.getSiteId(), JSONObject.toJSONString(enroll));
            return -1L;
        }
        enrollEntity.setUpdateById(context.getAccountId());
        enrollEntity.setUpdateByName(context.getAccountName());
        enrollEntity.setUpdateTime(new Date());
        enrollEntity.setCompanyId(context.getCompanyId());
        enrollEntity.setSiteId(context.getSiteId());

        enrollEntity.setTrainingProjectId(enroll.getTrainingProjectId());
        enrollEntity.setStartTime(enroll.getStartTime());
        enrollEntity.setEndTime(enroll.getEndTime());
        enrollEntity.setNeedAudit(enroll.getNeedAudit());
        enrollEntity.setPersonLimitNum(enroll.getPersonLimitNum());
        enrollEntity.setNotice(enroll.getNotice());
        Integer enablePay = enroll.getEnablePay();
        enrollEntity.setEnablePay(enablePay);
        if (null != enablePay && enablePay.equals(1)){
            //付费报名；无需审核
            enrollEntity.setNeedAudit(0);
            enrollEntity.setActualPrice(enroll.getActualPrice());
            enrollEntity.setOriginalPrice(enroll.getOriginalPrice());
        }
        enrollMapper.updateAllColumnById(enrollEntity);
        return enroll.getId();
    }

    @Override
    public Integer queryNeedAuditByTrainingProjectId(Long trainingProjectId) {
        Enroll selectOne = selectByProjectId(trainingProjectId);
        if (selectOne != null) {
            return selectOne.getNeedAudit();
        }
        return 0;
    }

    @Override
    public Enroll selectByProjectId(Long trainingProjectId) {
        List<Enroll> selectByMap = enrollMapper.selectByProjectId(trainingProjectId);
        if (selectByMap.size() > 0) {
            return selectByMap.get(0);
        }
        return null;
    }

    @Override
    public ReturnEnrollVO selectEnrollByProjectId(ProjectAccountVO vo) {
        Date date = new Date();
        ReturnEnrollVO rvo = new ReturnEnrollVO();
        Long projectId = vo.getProjectId();
        Long accountId = vo.getAccountId();
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("training_project_id", projectId);
        List<Enroll> enrollList = enrollMapper.selectByMap(map);
        if (enrollList.size() == 0 || enrollList == null) {
            rvo.setIsEnroll(0);
            rvo.setAuditStatus(7);
            return rvo;
        }
        Enroll enroll = enrollList.get(0);
        Date startTime = enroll.getStartTime();
        Date endTime = enroll.getEndTime();
        // 报名未开始
        if (date.before(startTime)) {
            rvo.setIsEnroll(0);
            rvo.setAuditStatus(4);
            return rvo;
        }
        // 报名已结束
        if (date.after(endTime)) {
            rvo.setIsEnroll(0);
            rvo.setAuditStatus(5);
            return rvo;
        }
        map.put("account_id", accountId);
        List<EnrollRecord> selectByMap = enrollRecordMapper.selectByMap(map);
        if (selectByMap == null || selectByMap.size() == 0) {
            rvo.setIsEnroll(EnrollConstant.ZERO);
            // 未报名
            rvo.setAuditStatus(6);
            return rvo;
        } else {
            EnrollRecord record = selectByMap.get(0);
            BeanUtils.copyProperties(record, rvo);
            if (record.getAuditStatus() == 0 || record.getAuditStatus() == 2) {
                rvo.setIsEnroll(1);
                rvo.setAuditStatus(2);
            } else if (record.getAuditStatus() == 1) {
                rvo.setIsEnroll(0);
                rvo.setAuditStatus(1);
            } else if (record.getAuditStatus() == 3) {
                rvo.setIsEnroll(0);
                rvo.setAuditStatus(3);
            }
            return rvo;
        }
    }

    @Override
    public GetEnrollVO getEnrollByProjectId(Long projectId, Long accountId) {
        Enroll enroll = new Enroll();
        enroll.setTrainingProjectId(projectId);
        enroll = enroll.selectOne(new EntityWrapper(enroll));
        // 如果查出了报名
        if (null != enroll) {
            // 报名基本信息
            GetEnrollVO vo = new GetEnrollVO();
            vo.setId(enroll.getId());
            vo.setLimit(enroll.getPersonLimitNum());
            vo.setNotice(enroll.getNotice());
            vo.setEndTime(enroll.getEndTime());
            vo.setStartTime(enroll.getStartTime());
            vo.setNeedAudit(enroll.getNeedAudit().equals(1));
            vo.setTrainingProjectId(projectId);
            //是否付费
            vo.setEnablePay(enroll.getEnablePay());
            vo.setActualPrice(enroll.getActualPrice());
            vo.setOriginalPrice(enroll.getOriginalPrice());
            // 已经报名人数
            EnrollRecord record = new EnrollRecord();
            record.setTrainingProjectId(projectId);
            EntityWrapper entityWrapper = new EntityWrapper(record);
            Byte []a = new Byte[]{0,2};
            entityWrapper.in("audit_status",a);
            vo.setHasEnrolledNum(record.selectCount(entityWrapper));
            // 当前用户报名状态
            record.setAccountId(accountId);
            record = record.selectOne(new EntityWrapper(record));
            if (record != null) {
                vo.setHasEnrolled(Boolean.valueOf(true));
                vo.setAuditStatus(record.getAuditStatus());
            }//若项目为限制人数类型，limitNum必不等于0
            if (enroll.getPersonLimitNum() != 0) {
                vo.setHasLimitNums(true);
            } else {
                vo.setHasLimitNums(false);
            }
            return vo;
        }
        return null;
    }

    @Override
    public Integer deleteEnrollByProjectId(ProjectAccountVO vo) {

        Long projectId = vo.getProjectId();
        try {
            if (null == projectId){
                logger.error("删除项目报名信息失败；传入项目id为空");
                return -1;
            }
            Enroll enroll = selectByProjectId(projectId);
            if (null == enroll){
                logger.info("删除项目报名-未查询到项目报名,不做删除操作;trainingProjectId = {}",projectId);
                return 1;
            }
            Enroll delWrapper = new Enroll();
            delWrapper.setTrainingProjectId(projectId);
            EntityWrapper<Enroll> en = new EntityWrapper<>(delWrapper);
            enrollMapper.delete(en);
            logger.info("删除报名成功; enroll = {}",JSONObject.toJSONString(enroll));
        }catch (Exception e){
            logger.error("删除项目报名信息异常: trainingProjectId = {} .异常信息：",projectId,e);
            return -1;
        }
        return 1;
    }
}